// Copyright 2023 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package cgotest

import "testing"

// Nothing to test here.
// The test is that the package compiles at all.
// See resstate.go.
func Test(t *testing.T) {
}
